<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\RelatCustomMovimentacaoService;
use HCM\Util\HCMParameters;

class RelatCustomMovimentacaoController extends Crud {

    CONST operatorLikeAll = 'LIKE_ALL';
    CONST operatorLike = 'LIKE';

    public function __construct (Manager $dataSourceManager, HCMParameters $hcmParameters, RelatCustomMovimentacaoService $relatCustomMovimentacaoService){
        $this->relatCustomMovimentacaoService = $relatCustomMovimentacaoService;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getRelatCustomMovimentacaoFinanceiro(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            ini_set('memory_limit', '2048M');
            $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'NRTPMODALIDCAL', 'NROCORRECAL', 'P_NRORG_PADRAO', 'NRTIPOHIERARQORG', 'NRESTRUTGEREN', 'NRTIPOESTRUTURA', 'DADOSVINCULO');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    if($filter['columnName'] === 'NROCORRECAL'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    if($filter['columnName'] === 'DADOSVINCULO'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    unset($filters[$key]);
                }
                if($filter['operator'] == self::operatorLikeAll){
                    $filters[0]['operator'] = self::operatorLike;
                    $filters[0]['columnName'] = str_replace('|', '||', $filter['columnName']);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $parameters['NROCORRECAL'] = $this->hcmParameters->getOcorrenciaCalculo();
            $parameters['NRTPMODALIDCAL'] = $this->hcmParameters->getNrModalidadeCalculo();
            
            if( !isset($parameters['DADOSVINCULO']) ){
                $parameters['DADOSVINCULO'] = "S";
            }
            
            $result = $this->relatCustomMovimentacaoService->getRelatCustomMovimentacaoFinanceiro($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRelatCustomMovimentacaoFinanceiro', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getRelatCustomMovimentacaoEventos(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            ini_set('memory_limit', '2048M');
            $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'NRTPMODALIDCAL', 'NROCORRECAL', 'P_NRORG_PADRAO', 'NRTIPOHIERARQORG', 'CDMATRICULA', 'CDMATRICULAESOCIAL', 'NRESTRUTLEGAL', 'NRESTRUTGEREN', 'NRESTRUTSIND', 'NRESTRUTSUPERIORM', 'NREVENTOM');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    if($filter['columnName'] === 'NROCORRECAL'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    unset($filters[$key]);
                }
                if($filter['operator'] == self::operatorLikeAll){
                    $filters[0]['operator'] = self::operatorLike;
                    $filters[0]['columnName'] = str_replace('|', '||', $filter['columnName']);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $result = $this->relatCustomMovimentacaoService->getRelatCustomMovimentacaoEventos($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRelatCustomMovimentacaoEventos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
} 