<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use Zeedhi\Framework\ORM\DateTime;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Repositories\FpaResumoctbl;

class FparesumoctblEntity extends TkEntity
{
    
    const TYPE_DAO = 'D';

    const TYPE_ENTITY = 'E';

    const TYPE_FACADE = 'F';

    const TYPE_LOGIC = 'L';

    const IDTIPOLANC_CREDITO = 'C';

    const IDTIPOLANC_DEBITO = 'D';
    
    public function __construct($entityManager, $hcmParameters) {
        $this->hcmParameters = $hcmParameters;
        $this->fpa_resumoCtbl = Repositories::FPA_RESUMOCTBL;
        parent::__construct(null,$entityManager);
    }
    
    public function retornaResumoContabilPorEstruturaMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMOCTBL)->retornaResumoContabilPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment
        );
        return $resultado;
    }

    public function retornaResumoContabilPorEstruturaSupMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMOCTBL)->retornaResumoContabilPorEstruturaSupMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment
        );
        return $resultado;
    }

    public function retornaResumoContabilPorEstruturasMesCompetencia($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMOCTBL)->retornaResumoContabilPorEstruturasMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment
        );
        return $resultado;
    }
    
    public function retornaResumoContabilPorEstruturaMesCompetenciaMovimento($nr_org, $dt_ocorrencia, $nr_tipomoviment){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMOCTBL)->retornaResumoContabilPorEstruturaMesCompetenciaMovimento(
            $nr_org, $dt_ocorrencia, $nr_tipomoviment
        );
        return $resultado;
    }
    
    public function insereResumoContabil(
        $nrorg, $dt_ocorrencia, $nr_estruturam, $idtipolanc, $cdcontctbl, $cdhistpadr, $vrlancamento, $nr_estrutura_sup = null
    ){
        try {
            
            $item = new $this->fpa_resumoCtbl;
            
            $item->setNrorg($nrorg);
            $item->setDtocorrencia($dt_ocorrencia);
            $item->setNrestruturamsup($nr_estrutura_sup);
            $item->setNrestruturam($nr_estruturam);
            $item->setIdtipolanc($idtipolanc);
            $item->setCdcontctbl($cdcontctbl);
            $item->setCdhistpadr($cdhistpadr);
            $item->setVrlancamento($vrlancamento);
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        
            $item->setNrresumoctbl(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RESUMOCTBL', 1, 12, true));

            $this->entityManager->persist($item);
            $this->entityManager->flush($item);
            
            return true;
        } catch(Exception $e){
            throw new Exception($e->getMessage());
            return false;
        }

    }

    public function criaListaResumoContabil($nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment){
        $resumo_contabil = $this->retornaResumoContabilPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura, $nr_tipomoviment
        );
        foreach ($resumo_contabil as $item) {
            $this->addItem($item, $item->getNrresumoctbl());
        }

        if (is_array($resumo_contabil) === true) {
            return true;
        } else {
            return false;
        }
    }

    public function criaListaResumoContabilPorEstruturaSup($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment){
        
        $resumo_contabil = $this->retornaResumoContabilPorEstruturaSupMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_tipomoviment
        );
        foreach ($resumo_contabil as $item) {
            $this->addItem($item, $item->getNrresumoctbl());
        }

        if (is_array($resumo_contabil) === true) {
            return true;
        } else {
            return false;
        }
    }

    public function criaListaResumoContabilPorEstruturas($nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment){
        
        $resumo_contabil = $this->retornaResumoContabilPorEstruturasMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura_sup, $nr_estrutura, $nr_tipomoviment
        );
        foreach ($resumo_contabil as $item) {
            $this->addItem($item, $item->getNrresumoctbl());
        }

        if (is_array($resumo_contabil) === true) {
            return true;
        } else {
            return false;
        }
    }
    
    public function criaListaResumoContabilLite($nr_org, $dt_ocorrencia, $nr_tipomoviment){
        $resumo_contabil = $this->retornaResumoContabilPorEstruturaMesCompetenciaMovimento(
            $nr_org, $dt_ocorrencia, $nr_tipomoviment
        );
        foreach ($resumo_contabil as $item) {
            $this->addItem($item, $item->getNrresumoctbl());
        }

        if (is_array($resumo_contabil) === true) {
            return true;
        } else {
            return false;
        }
    }

    public function procuraResumoContabilPorContaContabil($conta, $nrlancamento, $nrEstrutura, $historico, $idtipolanc){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            if ( ($conta === $item->getCdcontctbl())
               && ($nrlancamento === $item->getNrlancamento())
               && ($nrEstrutura === $item->getNrestruturam())
               && ($historico === $item->getCdhistpadr())
               && ($idtipolanc === $item->getIdtipolanc())
            ) {
                return $item;
            }
        }
    }

    public function atualizaListaResumoContabil(
        $nrorg, $dt_ocorrencia, $nr_estruturam, $idtipolanc, $conta_contabil, $historico, $valor_evento, $nr_estrutura_sup = null, $nrlancamento = null,
        $nr_tipomoviment = null
        
    ){
        
        $dt_ocorrencia = DateUtil::getDataDeString($dt_ocorrencia, DateUtil::FORMATO_BRASILEIRO, true);
        $item = $this->procuraResumoContabilPorContaContabil($conta_contabil, $nrlancamento, $nr_estruturam, $historico, $idtipolanc);

        if (is_object($item) === true) {
            $valor_lancamento = $item->getVrlancamento() + $valor_evento;
            $item->setNrorg($item->getNrorg());
            $item->setCdcontctbl($conta_contabil);
            $item->setCdhistpadr($historico);
            $item->setVrlancamento($valor_lancamento);
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        } else {
            $item = new $this->fpa_resumoCtbl;
            $item->setNrorg($nrorg);
            $item->setNrlancamento($nrlancamento);
            $item->setDtocorrencia($dt_ocorrencia);
            $item->setNrtipomoviment($nr_tipomoviment);
            $item->setNrestruturamsup($nr_estrutura_sup);
            $item->setNrestruturam($nr_estruturam);
            $item->setIdtipolanc($idtipolanc);
            $item->setCdcontctbl($conta_contabil);
            $item->setCdhistpadr($historico);
            $item->setVrlancamento($valor_evento);
            
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        
            $item->setNrresumoctbl(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RESUMOCTBL', 1, 12, true));

            $this->addItem($item, $item->getNrresumoctbl());
        }
    }
    
    public function removeItensZerados(){
        $itens = $this->getItems();
        if($itens){
            foreach ($itens as $item) {
                if($item->getVrlancamento()==0)
                    $this->removeItem($item);
            }
            return true;
        }else{
            return false;
        }    
    }
    
    public function retrocedeListaResumoContabil(){
        $itens = $this->getItems();
        if($itens){
            foreach ($itens as $item) {
                $this->removeItem($item);
            }
            return true;
        }else{
            return false;
        }  
    }

    public function getEventoSemAssociacaoPorSelecaoVinculos($nrorg, $nrorgpadrao, $dtmescompetenc, $nrcalculofolha, $dsordem, $dsinicio, $dsfim, $ativos=true, $rescindidos=true) {
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtmescompetenc));
        
        switch ($dsordem){
            case 'ALFABETICA':
                $condIntervalo = ' and vm.nmvinculom between \''.$dsinicio.'\' and \''.$dsfim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = ' and vh.nrestrutlegal = '.$dsinicio.'';
                if($dsfim)
                $condIntervalo .= "and vh.nrestrutgeren = '$dsfim'";
                break;
            case 'MATRICULA':
                $condIntervalo = ' and vm.cdmatricula between \''.$dsinicio.'\' and \''.$dsfim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = ' and vm.dtadmissaovinc between \''.$dsinicio.'\' and \''.$dsfim.'\'';
                break;
            case 'LISTA_VINCULO':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vm.nrvinculom IN ($dsinicio)";
                break;
            case 'LISTA_ESTRUTURA':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vh.nrestrutlegal IN ($dsinicio)";
                if($dsfim){
                    $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($dsfim)";
                }
                break;    
        }

        if ($ativos === true) {
            if ($rescindidos === true) {
                $condAtivos = ' and (vm.dtrescisaovinc is null
                                    or vm.dtrescisaovinc >= :p_dtmescompetenc
                                  )
								  and ((vm.dtadmissaovinc <> vm.dtrescisaovinc) or vm.dtrescisaovinc is null)
                                  and vm.dtadmissaovinc <=  :p_dtultdiames';
            } else {
                $condAtivos = ' and (vm.dtrescisaovinc is null
                                    or vm.dtrescisaovinc > :p_dtultdiames
                                  )
								  and ((vm.dtadmissaovinc <> vm.dtrescisaovinc) or vm.dtrescisaovinc is null)
                                  and vm.dtadmissaovinc <=  :p_dtultdiames';
            }
        }
        $resultado = $this->entityManager->getRepository(Repositories::FPA_RESUMOCTBL)->retornaEventoSemAssociacaoPorSelecaoVinculos($nrorg, $nrorgpadrao, $dtmescompetenc, $nrcalculofolha, $condIntervalo, $condAtivos);

        return $resultado;
    }
}