<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use ZipArchive;

class HcmManualService {
	const NRPARCNEGOCIO_DOCUMENTACAO = '191007531'; // '191006868' - HCM
	const NRORG_DOCUMENTACAO         = '0'; // '20';
    const DESCRIPTION_FULL           = "Tela: nmTela\r\nPágina: nmPagina\r\nNome do arquivo de manual: nmManual\r\n\nOperador: cdOper - nmOper\r\nOrgnaização: nrOrg"; //Vide controller do front
    const DESCRIPTION_PKVALUES       = "{\"NRORG\":\"nrOrg\", TELA: nmTela, PAGINA: nmPagina}";
    const INSERT_TKU_LOG             = 'insert into TKU_LOG (TKU_LOG_ID,CREATED,DESCRIPTION,TABLENAME,PKVALUES,DATADIFF,EXTENDED_DATA,IP,TYPE,USERNAME,NRORG) values (:TKU_LOG_ID,:CREATED,:DESCRIPTION,:TABLENAME,:PKVALUES,SUBSTR(:DATADIFF,1,2000),:EXTENDED_DATA,:IP,:TYPE,:USERNAME,:NRORG)';

    protected $entityManager;
    protected $nrorg;
    protected $cdoperador;
    protected $nmoperador;
    protected $tku_log_row = array('CREATED'=>null,'DESCRIPTION'=>null,'TABLENAME'=>null,'PKVALUES'=>null,'DATADIFF'=>null,'EXTENDED_DATA'=>null,'IP'=>null,'TYPE'=>null,'USERNAME'=>null,'NRORG'=>null);
    
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->nrorg         = $hcmParameters->getNrorgTrabalho();
        $this->cdoperador    = $hcmParameters->getCdoperador();
        $this->nmoperador    = $hcmParameters->getNmoperador();
    }
    
    
    private function getPathsManual($nmManual) {
        $nmManual .= '.';
        
        $params = array(
            'nrorg'             => self::NRORG_DOCUMENTACAO,
            'nrparcnegocio'     => self::NRPARCNEGOCIO_DOCUMENTACAO
            );
        $anexos = $this->entityManager->getRepository(Repositories::GPE_ANEXOS)->findBy($params);
        
        $paths = array();
        
        foreach($anexos as $anexo) {
            $nmanexo = strtolower($anexo->getNmanexo());
            $dspath  =            $anexo->getDspath();
            
            if (strpos($nmanexo, $nmManual) === 0) {
                $paths[] = array('nmanexo' => $nmanexo, 'dspath' => $dspath);
            }
        }
        
        return $paths;
    }
    
    
    private function getDescription($info, $source) {
        foreach($info as $key => $val) {
            $source = str_replace($key, $val, $source);
        }
        return $source;
    }
    
    
    private function setAdditionalInfo(&$info) {
        $info['nmOper'] = $this->nmoperador;
        $info['cdOper'] = $this->cdoperador;
        $info['nrOrg' ] = $this->nrorg;
    }
    
    
    private function setLog($info) {
        $this->setAdditionalInfo($info);
        $this->tku_log_row['TABLENAME'] = 'HCM_DOCUMENTACAO';
        $this->tku_log_row['DATADIFF' ] = $this->getDescription($info, self::DESCRIPTION_FULL);
        $this->tku_log_row['PKVALUES' ] = $this->getDescription($info, self::DESCRIPTION_PKVALUES);
        $this->tku_log_row['USERNAME' ] = $this->nmoperador;
        $this->tku_log_row['TYPE'     ] = "I";
        $this->tku_log_row['NRORG'    ] = $this->nrorg;
        
        try{
            $std = $this->entityManager->getConnection()->prepare('SELECT MAX(TKU_LOG_ID) + 1 AS TKU_LOG_ID FROM TKU_LOG');
            $std->execute();
            $result = $std->fetchAll();
            
            $this->tku_log_row['CREATED'   ] = date('d/m/Y H:i:s');
            $this->tku_log_row['IP'        ] = getHostByName(getHostName());
            $this->tku_log_row['TKU_LOG_ID'] = $result[0]['TKU_LOG_ID'];
            
            $this->entityManager->getConnection()->prepare(self::INSERT_TKU_LOG)->execute($this->tku_log_row);
        } catch (\Exception $e) {
            $error = "Error on log generation: ".$e->getMessage();
            throw new \Exception($error);
        }
    }
    
    public function getHcmManual($row) {
        $paths = $this->getPathsManual($row['nmManual']);

        $hasManyManuals = count($paths) > 1;
        $hasNoManual = empty($paths);
        
        if ($hasNoManual) {
            $this->setLog($row);

            throw new \Exception("HCM_MANUAL_NO_MANUAL");
        }

        if ($hasManyManuals) {
            $zipName = str_replace(".pdf", "", $paths[0]["nmanexo"]);

            foreach ($paths as $path) {
                $manualList[] = $path;
            }

            $this->zipManuals($manualList, $zipName);

            return '/tmp/hcm_mans/'.$zipName.'.zip';
        }

        return $paths;
    }

    public function zipManuals($manualList, $zipName) {
        $file = new File;
        $zip = new Zip;

        $file->cleanHcmManuals();

        $file->makeTmpDirectory();

        foreach ($manualList as $id => $manual) {
            $file->download($manual["dspath"]);

            $manual["nmanexo"] = str_replace(".pdf", "", $manual["nmanexo"]) . $id . ".pdf";

            rename(Directories::TMP.explode("/", $manual["dspath"])[2], Directories::TMP.$manual["nmanexo"]);
            
            $manualsToCompact[] = $manual["nmanexo"];
        }

        $zip->compactFiles($manualsToCompact, $zipName);

        foreach ($manualList as $id => $manual) {
            $manual["nmanexo"] = str_replace(".pdf", "", $manual["nmanexo"]) . $id . ".pdf";

            $file->cleanTmpFiles($manual["nmanexo"]);
        }
    }
}

class File {
    public function download($dspath) {
        $fh = fopen(Directories::TMP.explode("/", $dspath)[2], "w");
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, Util::getFileServerUrl() . $dspath);
        curl_setopt($ch, CURLOPT_FILE, $fh);
        curl_exec($ch);
        curl_close($ch);
    }

    public function makeTmpDirectory() {
        if (!file_exists(Directories::HCM_MANUALS)) {
            mkdir(Directories::HCM_MANUALS, 0777, true);
        }
    }

    public function cleanTmpFiles($nmanexo) {
        unlink(Directories::TMP.$nmanexo);
    }

    public function cleanHcmManuals() {
        $fileList = glob(Directories::HCM_MANUALS.'/*');
        
        foreach ($fileList as $filename) {
            if (is_file($filename)) {
                unlink($filename);
            }   
        }
    }
}

class Zip {
    public function compactFiles($fileList, $saveAs) {
        foreach ($fileList as $file) {
            $files[] = Directories::TMP.$file;
        }

        $zipName = '/tmp/hcm_mans/'.$saveAs.'.zip';

        $this->createZipArchive($files, $zipName);
    }

    private function createZipArchive($files = array(), $destination = '', $overwrite = false) {

        $fileAlreadyExists = file_exists($destination) && !$overwrite;
     
        $validFiles = $this->validFiles($files);

        $hasValidFilesToCompact = count($validFiles);

        if ($fileAlreadyExists) { 
            return false; 
        }
     
        if ($hasValidFilesToCompact) {
           $zip = new ZipArchive();

           if ($zip->open($destination,$overwrite ? ZIPARCHIVE::OVERWRITE : ZIPARCHIVE::CREATE)) {
              foreach ($validFiles as $file) {
                  $output = str_replace('/tmp/', '', $file);

                 $zip->addFile($file, $output);
              }
              
              $zip->close();
              return file_exists($destination);
           }
        }
        
        return false;
    }

    private function validFiles($files) {
        $validFiles = array();

        if (is_array($files)) {
            foreach ($files as $file) {
               if (file_exists($file)) {
                  $validFiles[] = $file;
               }
            }
        }

        return $validFiles;
    }
}

class Directories {
    const HCM_MANUALS = '/tmp/hcm_mans';
    const TMP = '/tmp/';
}